package Libconf::GUI::Gtk2;

use Libconf::GUI::Gtk2::Entry;
use Libconf::GUI::Gtk2::CheckButton;
use Libconf::GUI::Gtk2::Combo;
use Libconf::GUI::Gtk2::SpinButton;
use Libconf::GUI::Gtk2::Hostname;

our @ISA = qw(Exporter);
%EXPORT_TAGS = (
    generators => [ qw(generateWidget getBestWidget getDescription) ],
    builders => [ qw(buildWidget) ],
);
$EXPORT_TAGS{all} = [ map { @$_ } values %EXPORT_TAGS ];
@EXPORT_OK = map { @$_ } values %EXPORT_TAGS;

use lib qw(/usr/lib/libDrakX);
#use interactive;
use common;
use Libconf;
use ugtk2 qw(:create :dialogs :helpers :wrappers);

# public methods


sub get_description {
    my ($hash_desc, $hash_key) = @_;
    my $value;
    my $key = (keys(%$hash_key))[0];
    foreach (keys %$hash_desc) {
        $key =~ /^$_$/ and $value = $hash_desc->{$_}, last;
    }
    defined $value or return undef;
    my $value_desc = $hash_key->{$key};
    ref($value_desc) eq '' and return $value;
    if (ref($value_desc) eq 'ARRAY') {
        die "not yet implemented :( ";
    } elsif (ref($value_desc) eq 'HASH') {
        ref($value) eq 'HASH' or die;
        return get_description($value, $value_desc);
    }
}

# @widget_options is optional
sub getBestWidget {
    my ($struct, $key, @widget_options) = @_;
    my $data_description = $struct->{_data_description};
    my $data_mapping = $struct->{_data_mapping};
    my $hash = {};
    eval("\$hash->$key = 1;");
    $@ and die $@;
    my $ref_value = eval("\\\$struct->$key");
    my $ret = buildWidget($struct, $ref_value, get_description($data_description, $hash), $data_mapping, @widget_options);
    my $value = $$ref_value;
    defined $value ? $ret->set_value($value) : $ret->set_default();
    $ret;
}

sub getDescription {
    my ($struct, $key) = @_;
    my $data_description = $struct->{_data_description};
    my $hash = {};
    eval("\$hash->$key = 1;");
    $@ and die $@;
    my $data = get_description($data_description, $hash);
    defined $data or return undef;
    $data->{description};
}

# @widget_options is optional
sub buildWidget {
    my ($struct, $ref_value, $data, $data_mapping, @widget_options) = @_;
    my ($type, $type2, $type3, $default, $values) = @{$data}{qw(type type2 type3 default values)};
    $data_mapping ||= {};

    my $is_combo = 0;
    use Data::Dumper;
    Dumper($values) . "\n";
    if (defined $values) {
        if (ref($values) eq 'CODE') {
            $values = &$values();
        }
        if (ref($values) eq 'ARRAY') {
            $is_combo = 1;
        }
    }

    if (defined $default && ref($default) eq 'CODE') {
        $default = &$default($struct);
    }

    if ($type eq 'BOOLEAN') {
        $ret = Libconf::GUI::Gtk2::CheckButton->new($default, $ref_value, $data_mapping->{BOOLEAN}, @widget_options);
    } elsif ($type eq 'INTEGER') {
        $ret = Libconf::GUI::Gtk2::SpinButton->new($default, $ref_value, $data_mapping->{INTEGER}, @widget_options);
    } elsif ($type eq 'HOSTNAME') {
        $ret = Libconf::GUI::Gtk2::Hostname->new($default, $ref_value, $data_mapping->{HOSTNAME}, @widget_options);
    } elsif ($type eq 'STRING') {
        if ($is_combo) {
            $ret = gtkcombo_setpopdown_strings(Libconf::GUI::Gtk2::Combo->new($default, $ref_value, $data_mapping->{STRING}, @widget_options),
                                               @$values,
                                              );
        } else {
            $ret = Libconf::GUI::Gtk2::Entry->new($default, $ref_value, $data_mapping->{STRING}, @widget_options);
        }
    } else {
        $ret = Libconf::GUI::Gtk2::Entry->new($default, $ref_value, $data_mapping->{STRING}, @widget_options);
    }
    $ret;
}

#BOOLEAN
#COMMAND
#DIRECTORY
#DURATION
#ENCODING
#FILENAME
#INTEGER
#LIST
#PID
#STRING
#VERSION

sub generateWidget {
    my ($struct) = @_;
    my $vbox = Gtk2::VBox->new(0,10);
    my $notebook;
    my $n_page;
    my $table;
    if (UNIVERSAL::isa($struct, 'HASH')) {
        my $count = 0;
        while (my ($key, $value) = each %$struct) {
            if (UNIVERSAL::isa($value, 'HASH')) {
                if (!defined $notebook) {
                    $notebook = initialize_notebook($vbox);
                    $n_page = 0;
                }
                $notebook->append_page(
                                       my $child = generateWidget($value),
                                       gtkpack_(Gtk2::HBox->new(0,5),
                                                1, new Gtk2::Label($key),
                                                0, gtksignal_connect(# gtkadd(
                                                                     Gtk2::Button->new('X'),
                                                                     #gtkcreate_pixbuf('eth_card_mini2.png')
                                                                     #),
                                                                     clicked => sub {
                                                                         my $n_page = $_[1];
                                                                         if (warn_dialog(N("Warning"),
                                                                                         N("Do you really want to remove this section?"),
                                                                                         { cancel => 1 })) {
                                                                             $notebook->remove_page($n_page);
                                                                             delete $struct->{$key};
                                                                         }
                                                                     }, $n_page
                                                                    )
                                               ),
                                      );
                $n_page++;
            } else {
                $table ||= initialize_table($vbox);
                $table->attach(gtkpack_(Gtk2::HBox->new(0,0),
                                        1, Gtk2::HBox->new(0,0),
                                        0, Gtk2::Label->new($key)
                                       ),
                               0, 1, $count, $count + 1,
                               'fill',
                               ['shrink', 'shrink'],
                               0, 0);
                $table->attach(generateWidget($value), 1, 2, $count, $count + 1,
                               ['fill', 'expand'],
                               ['shrink', 'shrink'],
                               0, 0);
            }
            $count++;
        }
        $vbox->show_all();
        return $vbox;
    }
    if (UNIVERSAL::isa($struct, 'ARRAY')) {
        my $tree_model = Gtk2::TreeStore->new("Glib::String");
        my $list = Gtk2::TreeView->new_with_model($tree_model);
        $list->append_column(Gtk2::TreeViewColumn->new_with_attributes(N("list"), Gtk2::CellRendererText->new, 'text' => 0));
        foreach my $element (@$struct) {
            $tree_model->append_set(undef, [ 0 => $element ]);
        }
        $list->show_all();
        return $list;
    }
    return gtkshow(gtkentry($struct));
}

# private methods

sub initialize_notebook {
    gtkpack_(shift, 0, my $notebook = new Gtk2::Notebook);
    #        $notebook->set_tab_pos('left');
    $notebook;
}

sub initialize_table {
    gtkpack_(shift,
             0, gtkset_border_width(gtkset_shadow_type(gtkadd(Gtk2::Frame->new(N("test")),
                                                              my $table = Gtk2::Table->new(0, 2, 0),
                                                             ),
                                                       'etched_out'),
                                    5),
            );
    $table->set_col_spacings(5);
    $table->set_row_spacings(5);
    $table;
}

# sub create_dialog {
#     my ($label) = @_;
#     my $ret;
#     my $dialog = new Gtk2::Dialog();
#     $dialog->set_position('center');
#     $dialog->set_position('center-on-parent');
#     $dialog->vbox->set_border_width(10);
#     $dialog->signal_connect(delete_event => sub { Gtk2->main_quit() });
#     $dialog->vbox->pack_start(new Gtk2::Label($label),1,1,0);
#     my $bbox_dialog = new Gtk2::HButtonBox;
#     $dialog->action_area->add($bbox_dialog);
#     $bbox_dialog->set_layout('end');
#     $bbox_dialog->add(
#                       gtksignal_connect(new Gtk2::Button(N("OK")), 
#                                         clicked => sub {
#                                             $ret = 1;
#                                             Gtk2->main_quit();
#                                         })
#                      );
#     $bbox_dialog->add(
#                       gtksignal_connect(new Gtk2::Button(N("Cancel")), 
#                                         clicked => sub {
#                                             Gtk2->main_quit();
#                                         })
#                      );
#     $dialog->show_all;
#     $dialog->set_modal(1);
#     Gtk2->main();
#     $dialog->destroy;
#     $ret;
# }

1;

